(function (win, doc, exports) {

    'use strict';

    /**
     * 各部品（Button, Container, TextArea）のベースとなるクラス
     */
    function Component() {
        this.el = this._create();
    }
    Component.prototype = {

        /**
         * 空の div 要素を生成
         */
        _create: function () {
            return doc.createElement('div');
        },

        /**
         * 指定したクラスを要素に追加
         * className ... 追加したいクラス
         */
        addClass: function (className) {
            this.getEl().classList.add(className);
        },

        /**
         * 指定されたクラスを要素から削除
         * className ... 削除したいクラス
         */
        removeClass: function (className) {
            this.getEl().classList.remove(className);
        },

        /**
         * 指定されたコンポーネント自分自身に追加
         * cmp ... 対象のコンポーネント
         */
        append: function (cmp) {
            cmp.appendTo(this.getEl());
        },

        /**
         * 指定した要素へ自分を追加
         * target ... targetで指定された要素に自分を追加
         */
        appendTo: function (target) {
            target.appendChild(this.getEl());
        },

        /**
         * コンテナ要素を返す
         */
        getEl: function () {
            return this.el;
        },

        /**
         * コンテナ要素をDOMから削除し、参照も削除
         */
        destroy: function () {
            var el = this.getEl(),
                parent = el.parentNode;

            parent.removeChild(el);
            this.el = el = null;
        }
    };

    /*! ------------------------------------------------------------------------
        内部で定義したクラスを外部から参照できるようにExport
    ---------------------------------------------------------------------------- */
    exports.Component = Component;

}(window, document, window));
