(function (win, doc, exports) {

    'use strict';

    /**
     * カスタムダイアログを実装するクラス
     *     mes     ... カスタムダイアログで表示するメッセージ
     *     yesFunc ... YESボタンを押した時に実行される関数
     *     noFunc  ... NOボタンを押した時に実行される関数
     */
    function CustomDialog(mes, yesFunc, noFunc) {
        this._mes = mes;
        this._createElements();

        this._yesFunc = yesFunc || function () {};
        this._noFunc  = noFunc || function () {};

        doc.body.appendChild(this._container.getEl());
        this.hide();
    }
    CustomDialog.prototype = {
        /**
         * カスタムアラートを表示
         */
        show: function () {
            var container = this.getBody();

            setTimeout(function () {
                container.addClass('show');
            }, 20);
        },

        /**
         * カスタムアラートを非表示
         */
        hide: function () {
            var container = this.getBody();

            setTimeout(function () {
                container.removeClass('show');
            }, 20);
        },

        /**
         * カスタムアラートのコンテナを返す
         */
        getBody: function () {
            return this._container;
        },

        /**
         * カスタムアラートを構成する要素を生成
         *      container    ... カスタムアラートのウィンドウ部分
         *      btnContainer ... ボタングループ
         *      yesBtn       ... YESボタン
         *      noBtn        ... NOボタン
         *      textarea     ... 表示されるテキスト部分
         */
        _createElements: function () {

            var container    = new Container(),
                btnContainer = new Container(),
                yesBtn       = new Button('Yes'),
                noBtn        = new Button('No'),
                textarea     = new TextArea(this._mes);

            //ボタンコンテナにボタンを追加
            btnContainer.append(yesBtn);
            btnContainer.append(noBtn);
            btnContainer.addClass('btnGroup');

            //コンテナにテキストエリア、ボタングループを追加
            container.append(textarea);
            container.append(btnContainer);

            //各ボタンにイベントを設定
            yesBtn.click(this._onYes, this);
            noBtn.click(this._onNo, this);

            //各ボタンにクラスを追加
            yesBtn.addClass('btnYes');
            noBtn.addClass('btnNo');

            //カスタムアラートのコンポーネントにクラスを追加
            container.addClass('customDialog');

            this._container = container;
        },

        /**
         * YESボタンを押した時の動作（カスタムアラートを削除）
         */
        _onYes: function () {
            this._yesFunc();
            this.destroy();
        },

        /**
         * NOボタンを押した時の動作（カスタムアラートを削除）
         */
        _onNo: function () {
            this._noFunc();
            this.destroy();
        },

        /**
         * カスタムアラートを消去
         */
        destroy: function () {

            var self = this,
                el = this.getBody().getEl();

            function _destroy() {
                self._container.destroy();
                self._container = null;
                self._mes = null;
            }

            //削除前にいったん非表示に（CSSアニメーションあり）
            this.hide();

            //CSSのアニメーションが終了したのち、カスタムアラートをDOMツリーから削除
            el.addEventListener('webkitTransitionEnd', _destroy, false);
            el.addEventListener('MozTransitionEnd', _destroy, false);
            el.addEventListener('transitionend', _destroy, false);
        }
    };


    /*! ------------------------------------------------------------------------
        内部で定義したクラスを外部から参照できるようにExport
    ---------------------------------------------------------------------------- */
    exports.CustomDialog = CustomDialog;

}(window, document, window));
